//**************************************************************//
//  Name        : Analog In to 12-bit SPI DAC Microchip MCP4921 //
//  Author      : Gary Hill                                     //
//  Date        : 14 March, 2010                                //
//  Version     : 1.0                                           //
//  Reference(s): http://www.arduino.cc/playground/Code/Spi     //
//  Notes       : Code for reading analog input pin             //          
//              : and outputting to a 12-bit SPI DAC            //
//****************************************************************

// SPI Interface     SS_PIN(PB2), SCK_PIN(PB5), MOSI_PIN(PB3), MISO_PIN
// Arduino Pin       10           13            11             12
// MCP4921 DAC       SS           SCK           MOSI           n/a
#include <Spi.h>

// ATmega328P ADC 
#include "WProgram.h"
void setup();
void loop();
int analogPin = 0;     // analog input channel

// ADC analog input value
word sensorValue = 0;  // equivalent to unsigned int

// Byte of data to output to DAC 
byte data = 0;

void setup() {
  //set pin(s) to input and output
  pinMode(analogPin, INPUT);
}

void loop() {
  // read the value from the sensor:
  sensorValue = analogRead(analogPin);   // comment out this line to test DAC
  // sensorValue = 0x0200;               // 0x03FF = Vref, 0x0200 = 1/2 Vref, 0x0000 = 0
  sensorValue = sensorValue << 2 ;       // 10 bit ADC to 12-bit DAC word
  // set SS pin low, beginning 16-bit (2 byte) data transfer to DAC
  digitalWrite(SS_PIN, LOW);
  // send high byte
  data = highByte(sensorValue);  
  data = 0b00001111 & data;     // clear 4-bit command field (optional)  
  data = 0b00110000 | data;     // set command: 0 = DACA, 0 = buffered, 1 = 1x, 1 = output buffer enabled
  Spi.transfer(data);           // alternate: shiftOut(MOSI, SCK, MSBFIRST, data);
  // send low byte
  data = lowByte(sensorValue);
  Spi.transfer(data);           // alternate: shiftOut(MOSI, SCK, MSBFIRST, data);
  // set SS pin high, completing 16-bit transfer to DAC
  digitalWrite(SS_PIN, HIGH);
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

